/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_CONSTRAINT_H__
#define __TK_CONSTRAINT_H__

#include <gtkmm.h>

#include "libtkdb/database.h"
#include "libtkquery/query.h"

#include "editor/domain.h"

namespace Tk {
  namespace Editor {
    
    class Constraint : public Gtk::Entry
      {
      private:
	Tk::Db::Database *_database;
	Tk::String        _domain;
	bool              _valid;
	
	virtual void on_changed();
	void evaluate();
	void set_valid(bool valid, const Glib::ustring &tooltip = "");
	Tk::Query::Query *get_query(const Tk::String &query_string);
	
      public:
	typedef sigc::signal<void, const Tk::Db::Ids &> SignalConstraintActivated;
	
	SignalConstraintActivated signal_constraint_activated();
	
      protected:
	SignalConstraintActivated _signal_constraint_activated;
	virtual void on_activate();
	
      public:
	Constraint(Tk::Db::Database *database, Domain &domain);
	void set_domain(const Tk::String &domain);
	Tk::Query::Query *get_query();
      };
    
  } /* namespace Editor */
}; /* namespace Tk */

#endif /* __TK_CONSTRAINT_H__ */
