/*
 * tkresolver
 *
 * Copyright (C) 2004, 2005, 2006, 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkcommon/tkstring.h"

using namespace Tk;

String:: String(const mpq_class &q, unsigned long precision, size_type max_len)
{
  std::ostringstream o;
  o.precision(precision);
  o << static_cast<mpf_class>(q)
    << " ("
    << q 
    ;
  
  // clip string to ensure max_len characters
  std::string s(o.str());
  if (s.length() > max_len - 1)
    s = s.substr(0, max_len - 4) + "...";
  
  *this = s + ")";
}

StringList
StringList:: merge(const StringList &other) const
{
  StringList merged(*this);
  bool exists;
  
  for (StringList::const_iterator iter_val(other.begin());
       iter_val != other.end();
       ++iter_val)
    {
      exists = false;
      for (StringList::const_iterator iter_check(merged.begin());
	   iter_check != merged.end();
	   ++iter_check)
	{
	  if (*iter_val == *iter_check)
	    {
	      exists = true;
	      break;
	    }
	}

      if (!exists)
	merged.push_back(*iter_val);
    }

  return merged;
}

StringList
StringList:: uniq() const
{
  const static StringList null_list;
  return null_list.merge(*this);
}
