/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DECISION_TREE_H__
#define __TK_DECISION_TREE_H__

#include <wchar.h>

#include <iostream>

#include "randomness.h"
#include "random_types.h"

namespace Tk {

  namespace Random {

    class DecisionTree
    {
    private:
      class Node
      {
      private:
	Node *     _children[2];
	unsigned * _outcome;
      public:
	Node (unsigned outcomes, unsigned depth, unsigned &outcome);
	~Node ();

	unsigned get_outcome (Node * root, unsigned bit_descriptor, Randomness &rand, unsigned &retries) const;
      };

      unsigned _outcomes;
      unsigned _bit_multiplier;
      unsigned _depth;
      Node *   _root;
      unsigned _retries;

    public:
      DecisionTree (unsigned noutcomes, unsigned bit_multiplier);
      ~DecisionTree ();
      
      unsigned generate_event(Randomness &rand);
      
      unsigned get_depth() const;
      unsigned get_retries() const;
      unsigned tree_outcomes() const;
      double expected_tries() const;
      bool deterministic() const;
    };
    
  };
  
};


#endif /* __TK_DECISION_TREE_H__ */
