/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <wchar.h>

#include <stdlib.h>
#include <getopt.h>

#include "libtkcommon/debug.h"
#include "libtkcommon/numerical-types.h"
#include "libtkrandom/decision_tree.h"
#include "libtkrandom/retriever_registry.h"

using namespace Tk::Random;

void
test_decision_output(Randomness &rand, unsigned noutcomes, unsigned bit_multiplier, unsigned ntests)
{
  DecisionTree dt(noutcomes, bit_multiplier);
  
  std::map<unsigned, unsigned long> outcome_counts;
  unsigned long total_count;
  
  for (unsigned i = 0; i < ntests; ++i)
    {
      unsigned outcome( dt.generate_event(rand) );
      
      ++outcome_counts[outcome];
      ++total_count;
    }
  
  std::wcout << L"Outcomes: "
	     << noutcomes
	     << L"; bits: "
	     << bit_multiplier
	     << L"; tests: "
	     << ntests
	     << std::endl
	     << L"  ";
  for (std::map<unsigned, unsigned long>::iterator iter( outcome_counts.begin() );
       iter != outcome_counts.end();
       ++iter)
    {
      if (iter != outcome_counts.begin())
	std::wcout << L"; ";
      
      Tk::Real ratio(iter->second / total_count);
      std::wcout << iter->first
		 << L": "
		 << iter->second
		 << L" ("
		 << Tk::String(ratio)
		 << L")" ;
    }
  std::wcout << std::endl;
}

int
main (int argc, char ** argv)
{
  Randomness rand("alea", false, 64*1024, false);
  
  test_decision_output(rand, 3, 1, 10000);
  test_decision_output(rand, 5, 1, 10000);
  test_decision_output(rand, 7, 1, 10000);
  test_decision_output(rand, 3, 1, 1000000);
  test_decision_output(rand, 5, 1, 1000000);
  test_decision_output(rand, 7, 1, 1000000);
}
